=== modified file 'NEWS'
--- NEWS	2009-06-24 02:53:24 +0000
+++ NEWS	2009-06-25 09:19:36 +0000
@@ -131,9 +131,24 @@
 
     - Make sure the Atom feeds (nearly) validate. (Matt Nordhoff, #247162)
 
-    - Support serving branches over HTTP using the smart server protocol. 
+    - Support serving branches over HTTP using the smart server protocol.
       (Jelmer Vernooij, #306853)
 
+    - Serving branch data was broken when --allow-writes was *not*
+      passed. (Michael Hudson, #388730)
+
+    - http_serve config values are interpreted more forgivingly.
+      (Michael Hudson)
+
+    - When specifying a remote url to serve-branches, do not share
+      connections between threads. (Michael Hudson, #390972)
+
+    - http_serve values from locations.conf are now applied to
+      non-branch .bzr data (e.g shared repositories). (Michael Hudson)
+
+    - tags are now displayed. (Cris Boylan, Alexandre Garnier, Michael
+      Hudson, #246739)
+
     - Add a <meta> generator tag and display Loggerhead's version at the
       bottom of the page. (Matt Nordhoff, #370155)
 

=== modified file 'loggerhead/apps/transport.py'
--- loggerhead/apps/transport.py	2009-06-17 23:10:07 +0000
+++ loggerhead/apps/transport.py	2009-06-25 01:53:46 +0000
@@ -16,8 +16,11 @@
 #
 """Serve branches at urls that mimic a transport's file system layout."""
 
+import threading
+
 from bzrlib import branch, errors, lru_cache, urlutils
-from bzrlib.bzrdir import BzrDir
+from bzrlib.config import LocationConfig
+from bzrlib.transport import get_transport
 from bzrlib.transport.http import wsgi
 
 from paste.request import path_info_pop
@@ -26,9 +29,14 @@
 
 from loggerhead.apps.branch import BranchWSGIApp
 from loggerhead.apps import favicon_app, static_app
-from loggerhead.config import LoggerheadConfig
 from loggerhead.controllers.directory_ui import DirectoryUI
 
+_bools = {
+    'yes': True, 'no': False,
+    'on': True, 'off': False,
+    '1': True, '0': False,
+    'true': True, 'false': False,
+    }
 
 class BranchesFromTransportServer(object):
 
@@ -62,9 +70,8 @@
                 name = self.name
             else:
                 name = '/'
-            return DirectoryUI(environ['loggerhead.static.url'],
-                               self.transport,
-                               name)
+            return DirectoryUI(
+                environ['loggerhead.static.url'], self.transport, name)
         else:
             new_transport = self.transport.clone(segment)
             if self.name:
@@ -73,52 +80,70 @@
                 new_name = '/' + segment
             return BranchesFromTransportServer(new_transport, self.root, new_name)
 
+    def app_for_bazaar_data(self, relpath):
+        if relpath == '/.bzr/smart':
+            wsgi_app = wsgi.SmartWSGIApp(self.transport)
+            return wsgi.RelpathSetter(wsgi_app, '', 'PATH_INFO')
+        else:
+            # TODO: Use something here that uses the transport API
+            # rather than relying on the local filesystem API.
+            base = self.transport.base
+            readonly_prefix = 'readonly+'
+            if base.startswith(readonly_prefix):
+                base = base[len(readonly_prefix):]
+            try:
+                path = urlutils.local_path_from_url(base)
+            except errors.InvalidURL, e:
+                raise httpexceptions.HTTPNotFound()
+            else:
+                return urlparser.make_static(None, path)
+
+    def check_serveable(self, config):
+        value = config.get_user_option('http_serve')
+        if value is None:
+            return
+        elif not _bools.get(value.lower(), True):
+            raise httpexceptions.HTTPNotFound()
+
     def __call__(self, environ, start_response):
+        path = environ['PATH_INFO']
         try:
             b = branch.Branch.open_from_transport(self.transport)
         except errors.NotBranchError:
+            if path.startswith('/.bzr'):
+                self.check_serveable(LocationConfig(self.transport.base))
+                return self.app_for_bazaar_data(path)(environ, start_response)
             if not self.transport.listable() or not self.transport.has('.'):
                 raise httpexceptions.HTTPNotFound()
             return self.app_for_non_branch(environ)(environ, start_response)
         else:
-            if b.get_config().get_user_option('http_serve') == 'False':
-                raise httpexceptions.HTTPNotFound()
+            self.check_serveable(b.get_config())
+            if path.startswith('/.bzr'):
+                return self.app_for_bazaar_data(path)(environ, start_response)
             else:
                 return self.app_for_branch(b)(environ, start_response)
 
 
+_transport_store = threading.local()
+
+def get_transport_for_thread(base):
+    """ """
+    thread_transports = getattr(_transport_store, 'transports', None)
+    if thread_transports is None:
+        thread_transports = _transport_store.transports = {}
+    if base in thread_transports:
+        return thread_transports[base]
+    transport = get_transport(base)
+    return transport
+
+
 class BranchesFromTransportRoot(object):
 
-    def __init__(self, transport, config):
+    def __init__(self, base, config):
         self.graph_cache = lru_cache.LRUCache(10)
-        self.transport = transport
-        wsgi_app = wsgi.SmartWSGIApp(self.transport)
-        self.smart_server_app = wsgi.RelpathSetter(wsgi_app, '', 'PATH_INFO')
+        self.base = base
         self._config = config
 
-    def get_local_path(self):
-        """Raise exception if it's not a local path, otherwise return it"""
-
-        # TODO: Use something here that uses the transport API 
-        # rather than relying on the local filesystem API.
-        try:
-            path = urlutils.local_path_from_url(self.transport.base)
-        except errors.InvalidURL:
-            raise httpexceptions.HTTPNotFound()
-        else:
-            return path
-
-    def check_is_a_branch(self, path_info):
-        """Check if it's a branch, and that it's allowed to be shown"""
-        try:
-            bzrdir = BzrDir.open_containing_from_transport(
-                       self.transport.clone(path_info))[0]
-            branch = bzrdir.open_branch()
-            if branch.get_config().get_user_option('http_serve') == 'False':
-                raise httpexceptions.HTTPNotFound()
-        except errors.NotBranchError:
-            return
-
     def __call__(self, environ, start_response):
         environ['loggerhead.static.url'] = environ['SCRIPT_NAME']
         if environ['PATH_INFO'].startswith('/static/'):
@@ -127,24 +152,17 @@
             return static_app(environ, start_response)
         elif environ['PATH_INFO'] == '/favicon.ico':
             return favicon_app(environ, start_response)
-        elif environ['PATH_INFO'].endswith("/.bzr/smart"):
-            self.check_is_a_branch(environ['PATH_INFO'])
-            return self.smart_server_app(environ, start_response)
-        elif '/.bzr/' in environ['PATH_INFO']:
-            self.check_is_a_branch(environ['PATH_INFO'])
-            path = self.get_local_path()
-            app = urlparser.make_static(None, path)
-            return app(environ, start_response)
         else:
+            transport = get_transport_for_thread(self.base)
             return BranchesFromTransportServer(
-                self.transport, self)(environ, start_response)
+                transport, self)(environ, start_response)
 
 
 class UserBranchesFromTransportRoot(object):
 
-    def __init__(self, transport, config):
+    def __init__(self, base, config):
         self.graph_cache = lru_cache.LRUCache(10)
-        self.transport = transport
+        self.base = base
         self._config = config
         self.trunk_dir = config.get_option('trunk_dir')
 
@@ -158,13 +176,15 @@
         elif path_info == '/favicon.ico':
             return favicon_app(environ, start_response)
         else:
+            transport = get_transport_for_thread(self.base)
             # segments starting with ~ are user branches
             if path_info.startswith('/~'):
                 segment = path_info_pop(environ)
-                new_transport = self.transport.clone(segment[1:])
+                new_transport = transport.clone(segment[1:])
                 return BranchesFromTransportServer(
-                    new_transport, self, segment)(environ, start_response)
+                    transport.clone(segment[1:]), self, segment)(
+                    environ, start_response)
             else:
-                new_transport = self.transport.clone(self.trunk_dir)
                 return BranchesFromTransportServer(
-                    new_transport, self)(environ, start_response)
+                    transport.clone(self.trunk_dir), self)(
+                    environ, start_response)

=== modified file 'loggerhead/controllers/changelog_ui.py'
--- loggerhead/controllers/changelog_ui.py	2009-05-29 13:39:28 +0000
+++ loggerhead/controllers/changelog_ui.py	2009-06-25 03:35:43 +0000
@@ -89,9 +89,16 @@
                 self._branch.is_root,
                 'changes'))
 
+        show_tag_col = False
+        for change in changes:
+            if change.tags is not None:
+                show_tag_col = True
+                break
+
         return {
             'branch': self._branch,
             'changes': changes,
+            'show_tag_col': show_tag_col,
             'data': simplejson.dumps(data),
             'util': util,
             'history': history,

=== modified file 'loggerhead/history.py'
--- loggerhead/history.py	2009-06-24 00:59:24 +0000
+++ loggerhead/history.py	2009-06-25 03:39:27 +0000
@@ -654,6 +654,13 @@
 
         message, short_message = clean_message(revision.message)
 
+
+        tags = self._branch.tags.get_reverse_tag_dict()
+
+        revtags = None
+        if tags.has_key(revision.revision_id):
+          revtags = ', '.join(tags[revision.revision_id])
+
         entry = {
             'revid': revision.revision_id,
             'date': datetime.datetime.fromtimestamp(revision.timestamp),
@@ -665,6 +672,7 @@
             'comment_clean': [util.html_clean(s) for s in message],
             'parents': revision.parent_ids,
             'bugs': [bug.split()[0] for bug in revision.properties.get('bugs', '').splitlines()],
+            'tags': revtags,
         }
         return util.Container(entry)
 

=== renamed file 'serve-branches' => 'loggerhead/main.py' (properties changed: +x to -x)
--- serve-branches	2009-06-05 01:59:11 +0000
+++ loggerhead/main.py	2009-06-25 01:48:58 +0000
@@ -37,7 +37,7 @@
 from loggerhead.apps.error import ErrorHandlerApp
 
 
-def main(args):
+def get_config_and_path(args):
     config = LoggerheadConfig()
 
     if config.get_option('show_version'):
@@ -48,39 +48,20 @@
         config.print_help()
         sys.exit(1)
     elif config.arg_count == 1:
-        path = config.get_arg(0)
-    else:
-        path = '.'
-
-    load_plugins()
-
-    if config.get_option('allow_writes'):
-        transport = get_transport(path)
-    else:
-        transport = get_transport('readonly+' + path)
-
-    if config.get_option('trunk_dir') and not config.get_option('user_dirs'):
-        print "--trunk-dir is only valid with --user-dirs"
-        sys.exit(1)
-
-    if config.get_option('reload'):
-        if Reloader.is_installed():
-            Reloader.install()
-        else:
-            return Reloader.restart_with_reloader()
-
-    if config.get_option('user_dirs'):
-        if not config.get_option('trunk_dir'):
-            print "You didn't specify a directory for the trunk directories."
-            sys.exit(1)
-        app = UserBranchesFromTransportRoot(transport, config)
-    else:
-        app = BranchesFromTransportRoot(transport, config)
-
-    # setup_logging()
+        base = config.get_arg(0)
+    else:
+        base = '.'
+
+    if not config.get_option('allow_writes'):
+        base = 'readonly+' + base
+
+    return config, base
+
+
+def setup_logging(config):
     logging.basicConfig()
     logging.getLogger('').setLevel(logging.DEBUG)
-    logger = getattr(app, 'log', logging.getLogger('loggerhead'))
+    logger = logging.getLogger('loggerhead')
     if config.get_option('log_folder'):
         logfile_path = os.path.join(
             config.get_option('log_folder'), 'serve-branches.log')
@@ -92,8 +73,29 @@
     logfile.setFormatter(formatter)
     logfile.setLevel(logging.DEBUG)
     logger.addHandler(logfile)
-
-    # setup_logging() #end
+    return logger
+
+
+def make_app_for_config_and_path(config, base):
+    if config.get_option('trunk_dir') and not config.get_option('user_dirs'):
+        print "--trunk-dir is only valid with --user-dirs"
+        sys.exit(1)
+
+    if config.get_option('reload'):
+        if Reloader.is_installed():
+            Reloader.install()
+        else:
+            return Reloader.restart_with_reloader()
+
+    if config.get_option('user_dirs'):
+        if not config.get_option('trunk_dir'):
+            print "You didn't specify a directory for the trunk directories."
+            sys.exit(1)
+        app = UserBranchesFromTransportRoot(base, config)
+    else:
+        app = BranchesFromTransportRoot(base, config)
+
+    setup_logging(config)
 
     if config.get_option('profile'):
         from loggerhead.middleware.profile import LSProfMiddleware
@@ -129,7 +131,17 @@
 
     app = HTTPExceptionHandler(app)
     app = ErrorHandlerApp(app)
-    app = TransLogger(app, logger=logger)
+    app = TransLogger(app, logger=logging.getLogger('loggerhead'))
+
+    return app
+
+
+def main(args):
+    load_plugins()
+
+    config, path = get_config_and_path(args)
+
+    app = make_app_for_config_and_path(config, path)
 
     if not config.get_option('user_port'):
         port = '8080'
@@ -142,7 +154,3 @@
         host = config.get_option('user_host')
 
     httpserver.serve(app, host=host, port=port)
-
-
-if __name__ == "__main__":
-    main(sys.argv)

=== modified file 'loggerhead/static/css/diff.css'
--- loggerhead/static/css/diff.css	2009-05-30 17:40:05 +0000
+++ loggerhead/static/css/diff.css	2009-06-17 20:28:20 +0000
@@ -31,7 +31,7 @@
     width:493px;
     margin:0px 0px 5px 0px;
 }
-.information, .committer, .timer, .mfrom, .mto, .revid, .bug {
+.information, .committer, .timer, .mfrom, .mto, .revid, .bug, .tags {
     clear:both;
     margin:0px;
     padding:0px 0px 2px 18px;
@@ -43,6 +43,11 @@
     padding:1px 0px 0px 18px;
     margin:0px 10px 5px 0px;
 }
+.tags {
+    padding:1px 0px 0px 18px;
+    margin:0px 10px 5px 0px;
+    background:url(../images/ico_tag.gif) no-repeat top left;
+}
 .committer, .mfrom, .mto {
     color:#000;
     padding:1px 0px 0px 18px;

=== modified file 'loggerhead/static/css/files.css'
--- loggerhead/static/css/files.css	2009-05-28 22:48:45 +0000
+++ loggerhead/static/css/files.css	2009-06-17 20:28:20 +0000
@@ -1,5 +1,5 @@
 /*Info Box*/
-.information, .committer, .timer, .mfrom, .mto, .revid, .bug {
+.information, .committer, .timer, .mfrom, .mto, .revid, .bug, .tags {
 	float:left;
 	margin:0px 10px 5px 0px;
 	padding:0px 0px 2px 18px;
@@ -12,6 +12,12 @@
     padding:1px 0px 0px 18px;
     margin:0px 10px 5px 0px;
 }
+.tags {
+    float:left;
+    padding:1px 0px 0px 18px;
+    margin:0px 10px 5px 0px;
+    background:url(../images/ico_tag.gif) no-repeat top left;
+}
 .comiter, .mfrom, .mto {
 	color:#000;
 	padding:1px 0px 0px 18px;

=== modified file 'loggerhead/static/css/global.css'
--- loggerhead/static/css/global.css	2009-03-05 22:56:59 +0000
+++ loggerhead/static/css/global.css	2009-06-11 21:16:27 +0000
@@ -230,13 +230,19 @@
 .expandcell, .expcell {
 	width:28px;	
 }
-.summarycell, .authorcell, .datecell, .timedate{
+.summarycell, .authorcell, .tagscell, .datecell, .timedate{
   padding: 4px 8px;
   border: 1px solid #d2d2d2;
 }
 .icocell {
   width: 18px;
 }
+.authorcell, .autcell {
+  width: 240px;
+}
+.tagscell, .tagcell {
+  width: 240px;
+}
 .datecell, .date {
   width: 180px;
 }
@@ -246,7 +252,7 @@
 .downloadcell, .downr {
 	width:47px;
 }
-.revnro, .expcell, .diffr, .summcell, .autcell, .icocell, .date, .downr, .allicones, .timedate2 {
+.revnro, .expcell, .diffr, .summcell, .autcell, .tagcell, .icocell, .date, .downr, .allicones, .timedate2 {
   border: 0;
   padding: 4px 4px;
   border-bottom:1px solid #d2d2d2;
@@ -345,4 +351,4 @@
 }
 .revision_log {
     overflow: hidden;
-}
\ No newline at end of file
+}

=== added file 'loggerhead/static/images/ico_tag.gif'
Binary files loggerhead/static/images/ico_tag.gif and loggerhead/static/images/ico_tag.gif differ
=== modified file 'loggerhead/templates/changelog.pt'
--- loggerhead/templates/changelog.pt	2009-03-31 21:15:48 +0000
+++ loggerhead/templates/changelog.pt	2009-06-25 03:35:43 +0000
@@ -73,6 +73,7 @@
           <td class="expandcell show_if_js">&nbsp;</td>
           <td class="summarycell">Summary</td>
           <td class="authorcell">Authors</td>
+          <td tal:condition="show_tag_col" class="tagscell">Tags</td>
           <td class="datecell">Date</td>
           <td class="diffcell">Diff</td>
           <td class="downloadcell">Files</td>
@@ -111,6 +112,8 @@
             </td>
             <td tal:content="python:util.trunc(util.hide_email(entry.authors[0]), 20)"
                 class="autcell"></td>
+            <td tal:condition="show_tag_col" tal:content="string:${entry/tags}"
+                class="tagcell"></td>
             <td class="date">
               <span tal:attributes="title python:util.date_time(entry.date)"
                     tal:content="python:util._approximatedate(entry.date)"></span>

=== modified file 'loggerhead/templates/revisioninfo.pt'
--- loggerhead/templates/revisioninfo.pt	2009-05-30 17:40:05 +0000
+++ loggerhead/templates/revisioninfo.pt	2009-06-11 21:16:27 +0000
@@ -18,6 +18,7 @@
             <tal:b content="merged_in"/></a>.
         </div>
         <div class="revid" tal:content="string:Revision ID: ${change/revid}"></div>
+        <div class="tags" tal:condition="change/tags" tal:content="string:Tags: ${change/tags}"></div>
         <div class="clear"><!-- --></div>
         <div class="bug" tal:repeat="bug change/bugs">
             <a tal:attributes="href bug" tal:content="bug"></a>

=== added file 'serve-branches'
--- serve-branches	1970-01-01 00:00:00 +0000
+++ serve-branches	2009-06-24 03:51:44 +0000
@@ -0,0 +1,27 @@
+#!/usr/bin/env python
+#
+# Copyright (C) 2008, 2009 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Search for branches underneath a directory and serve them all."""
+
+import sys
+
+from loggerhead.main import main
+
+
+if __name__ == "__main__":
+    main(sys.argv)

